var Tabler = require('./lib/tabler'),
    tabler = new Tabler(
        'jsonfile', // Easiest backend to use for illustration, would work with any
        {
            filename: 'example.json'
        }
    ),
    pid1,
    pid2,
    fs = require('fs');
        

// Define a table of players (some sort of game)
// Each player has a username, a join date, a login counter, and a coordinate
// location within the game of the form {x: 0, y: 0}
tabler.defineTable(
    'Players',
    {
        username: {
            type:       'string',
            byteLength: 30,
            pattern:    /^[a-zA-Z0-9]+/
        },
        joined: {
            type:       'datetime'
        },
        logins: {
            type:       'number',
            min:        0,
            max:        999999,
            decPlaces:  0
        },
        coords: {
            type:       'object',
            byteLength: 50,
            schema: { // Require objects of the form {x: 1, y: 1} (http://tools.ietf.org/html/draft-zyp-json-schema-02)
                properties: {
                    x: {
                        type: 'number',
                        required: true
                    },
                    y: {
                        type: 'number',
                        required: true
                    }
                },
                additionalProperties: false
            }
        }
    
    }, function (error) {
        
        // Success
        step2();
        
    }
);

function step2 () {

    // Create rows for two players
    tabler.insertMulti(
        'Players',
        [
            {
                username:   'Joe123',       // Schema-valid
                logins:     0,              // Schema-valid
                joined:     new Date(),     // Schema-valid
                coords:     {x: 0, y: 0}    // Schema-valid
            },
            {
                username:   'Bill99',       // Schema-valid
                logins:     0,              // Schema-valid
                joined:     new Date(),     // Schema-valid
                coords:     {x: 1, y: 1}    // Schema-valid
            }
        ],
        function (error, rowIds) {
        
            // Success -- we now have the player row ids
            pid1 = rowIds[0];
            pid2 = rowIds[1];
            step3();
            
        }
    );

};

function step3 () {

    // Get the rows for those two players
    tabler.selectMulti(
        'Players',
        null,   // Get all fields
        [pid1, pid2], // 
        function (error, rows) {
            
            // Success
            // rows[0] is Joe123
            // rows[1] is Bill99
            console.log(rows);
            step4();
        
        }
    );

};

function step4 () {

    // Update Joe123's logins
    tabler.updateQuery(
        'Players',
        {
            logins: 1
        },
        [
            ['username', '=', 'Joe123']
        ],
        function (error) {
            
            // Success
            step5();
        
        }
    );

};


function step5 () {

    // Retrieve the player with the most logins
    tabler.selectQuery(
        'Players',
        null,   // Get all fields
        [],     // No conditions -> all players
        {
            orderBy:    'logins',
            orderDir:   'desc',
            limit:      1
        },
        function (error, rows) {
            
            // Success, one row returned
            // rows[0] is Joe123
            console.log(rows);
            step6();
        
        }
    );

};

function step6 () {

    // Remove the table
    tabler.deleteTable(
        'Players',
        function (error) {
            
            // Success
            fs.unlinkSync('example.json');
            console.log('done');
        
        }
    );

};